#!/usr/bin/python -d

import os
import sys
import re
import xmlrpclib
import getopt
from bsddb3 import db
from nrh_pkglist_op import process_pkglist
from nrh_header_lib import *
from nrh_rpm_lib import *

try:
	import rpm404
	rpm=rpm404
except:
	import rpm

class locate_latest_pkglist:
	def run(self,optlist,argv):

		dirList = os.listdir('./')
		latestFileName = ''

		for file in dirList:
			if re.search('^redhat-linux-i386-.*\.(\d{14})$', file):
				if not re.search('-obsoletes', file):
					print 'Found package list', file, ', timestamp :', file[-14:]
					if latestFileName[-14:] < file[-14:]:
						latestFileName = file

		if latestFileName == '':
			print 'No package lists found'
		else:
			print "The latest package list is", latestFileName
			try:
				argv[0] = latestFileName
				print 'Error, no parameters should be supplied while searching for latest package list'
				sys.exit()
			except IndexError:
				argv.append(latestFileName)

class update_repository:
	def run(self,optlist,argv):
		try:
			pkglistfile = argv[0]
		except:
			print "Error : please supply package list filename as command line parameter."
			sys.exit()

		PARTIAL=0

		for i in optlist:
			if i[0] == "--partial":
				PARTIAL=1

		listdate = pkglistfile[-14:]
		channelname = pkglistfile[:-15]

		print 'Updating channel repository',channelname,',package list generated at',listdate,

		if PARTIAL:
			print ',assuming partial channel'
		else:
			print

		# We first name it provides-list.temp.[timestamp].db, then after 
		# updating the time stamp in nrh-list file, we rename this db to
		# appropriate name with out the temp flag
		tempprovidesdb="provides-list.temp.%s.db"%listdate
		providesdb="provides-list.%s.db"%listdate

		# first make sure we remove the old provides database
		if os.path.isfile(tempprovidesdb): os.remove(tempprovidesdb)

		# Then open up new one
		reposDB=db.DB()
		reposDB.set_flags(db.DB_DUP)
		try:
			reposDB.open(tempprovidesdb, None, db.DB_BTREE, db.DB_CREATE, mode=0644)
		except:
			print "Cannot open provides database for creation\n"
			sys.exit(1)

		# We open up the pkglistfile and the iterate until the end of the file
		try:
			f = open(pkglistfile, "r")
			filecontents = f.read()
		except:
			print 'Cannot open file', pkglistfile, 'for reading'
			print 'Please make sure that you have given right file name as argument'
			sys.exit (1)

		try:
			pkglist, tmp_method = xmlrpclib.loads(filecontents)
		except NameError, e:
			print 'An rror occured while parsing package list'
			sys.exit(1)

		for pkg in pkglist[0]:

			# Now put together the full RPM name
			currentpkg="%s-%s-%s.%s.rpm"%(pkg[0],pkg[1],pkg[2],pkg[4])

			#if os.path.isfile(currentpkg):
			if rpmOps().Exists(currentpkg):

				# if its appropriate header does not exist, create it
				localHeader=rpmHeader()
				if not localHeader.Exists(currentpkg):
					# We have to wrap currentpkg (which is string object) into 
					# a list of objects since header lib's Create method only expects
					# list of objects.  This is a bit ugly, but convenient work around.
					argList=[]
					argList.append(currentpkg)
					localHeader.Create(argList)

				print currentpkg, ':',

				# Let's check whether this package exists in provides-list.db
				# if it doesn't, put it in
				data=reposDB.get_both(pkg[0],"%s\t%s\t%s\t%s"%(pkg[0],pkg[1],pkg[2],pkg[3]))
				if not data: reposDB.put(pkg[0], "%s\t%s\t%s\t%s"%(pkg[0],pkg[1],pkg[2],pkg[3]))

				try:
					fd = os.open(currentpkg,0)
				except:
					print "Cannot open %s for reading...\nExiting\n"
					sys.exit(1)
					
				(header,isSource) = rpm.headerFromPackage(fd)
				os.close(fd)

				# Now that we have the header, we can figure out what it provides
				#   and the files it provides
				print 'dependencies ... ',
				for dep in header[rpm.RPMTAG_PROVIDES]:
					data=reposDB.get_both(dep,"%s\t%s\t%s\t%s"%(pkg[0],pkg[1],pkg[2],pkg[3]))
					if not data: reposDB.put(dep,"%s\t%s\t%s\t%s"%(pkg[0],pkg[1],pkg[2],pkg[3]))

				# We also have to generate the list of files this specific RPM will
				#   install and put that information in the database
				print 'files ... ',
				for filename in header['filenames']:
					data=reposDB.get_both(filename,"%s\t%s\t%s\t%s"%(pkg[0],pkg[1],pkg[2],pkg[3]))
					if not data: reposDB.put(filename,"%s\t%s\t%s\t%s"%(pkg[0],pkg[1],pkg[2],pkg[3]))
				print "Done"
			elif not PARTIAL:
				print "%s is missing. If you are not fully mirroring this channel, use --partial"%currentpkg

		reposDB.close()

		#Copy the list to listPackages/<listdate>
		if os.path.isdir('listPackages'):
			for i in os.listdir('listPackages'):
				print "Unlinking old package list listPackages/%s"%i
				try:
					os.remove('listPackages/%s'%i)
				except:
					print "Could not remove listPackages/%s\nExiting..."%i
					sys.exit(1)
		else:
			try:
				os.mkdir('listPackages',0755)
			except:
				print "Cannot create listPackages directory\nExiting..."
				sys.exit(1)

		# Copy the obsoletes list to getObsoletes/<listdate>
		if os.path.isdir('getObsoletes'):
			for i in os.listdir('getObsoletes'):
				print "Unlinking old package list getObsoletes/%s"%i
				try:
					os.remove('getObsoletes/%s'%i)
				except:
					print "Could not remove getObsoletes/%s\nExiting..."%i
					sys.exit(1)
		else:
			try:
				os.mkdir('getObsoletes',0755)
			except:
				print "Cannot create getObsoletes directory\nExiting..."
				sys.exit(1)

		tmpdirname1=pkglistfile
		tmpdirname2="listPackages/%s"%listdate
		try:
			process_pkglist().run(tmpdirname1,tmpdirname2)
		except:
			print "Fatal error updating package list\nExiting..."
			sys.exit(1)

		tmpdirname1="%s-obsoletes.%s"%(channelname,listdate)
		tmpdirname2="getObsoletes/%s"%listdate
		try:
			process_pkglist().run(tmpdirname1,tmpdirname2)
		except:
			print "Fatal error updating package list\nExiting..."
			sys.exit(1)

		try:
			fd=open("nrh-listdate","w")
		except:
			print "Could not open nrh-listdate for writing\nExiting..."
			sys.exit(1)

		fd.write("%s"%listdate)
		os.rename(tempprovidesdb,providesdb)

		fd.close()

class clean_repository:
	def run(self,optlist,argv):
		try:
			pkglistfile = argv[0]
		except:
			print "Error : please supply package list filename as command line parameter."
			sys.exit(0)

		CONFIRM=1

		for i in optlist:
			if i[0] == "--yes":
				CONFIRM=0

		print 'Cleaning up channel repository according to package listfile',pkglistfile

		pkgList = ['listPackages','getObsoletes','nrh-listdate']

		goodFiles = 0

		if not os.access(pkglistfile, os.R_OK):
			print "Fatal error : package list file does not exist"
			sys.exit()

		listdate = pkglistfile[-14:]
		channelname = pkglistfile[:-15]

		pkgList.append(pkglistfile)
		pkgList.append("%s-obsoletes.%s"%(channelname,listdate))

		pkgList.append("applet-errata.%s"%listdate)
		pkgList.append("applet-errata.%s.zlib"%listdate)

		pkgList.append("provides-list.%s.db"%listdate)

		# FIXME error handling and what not
		f = open(pkglistfile, "r")
		filecontents = f.read()

		try:
			tmp_args, tmp_method = xmlrpclib.loads(filecontents)
		except:
			# if there was an error decoding return as if we didnt find it
			# the odd thing is, in testing, it's actually pretty hard to get
			# a file to show as corrupt, I think perhaps the xmlrpclib parser
			# is a bit too lenient...
			print "File is corrupt"
			sys.exit()

		dirList = os.listdir('./')

		for pkg in tmp_args[0]:
			pkgList.append("%s-%s-%s.%s.hdr" % (pkg[0], pkg[1], pkg[2], pkg[4]))
			pkgList.append("%s-%s-%s.%s.rpm" % (pkg[0], pkg[1], pkg[2], pkg[4]))

		delList = []

		for file in dirList:
			if file in pkgList:
				goodFiles = goodFiles + 1
			else:
				print file, 'is not in package list, queuing file for deletion'
				delList.append(file)

		if not (delList == []):
			if CONFIRM == 0:
				print 'Deleting files, please wait ...'
				for file in delList:
					os.unlink(file)
			else:
				if raw_input('Would you like to proceed deleting these files (y\\n) : ') == 'y':
					print 'Deleting files, please wait ...'
					for file in delList:
						os.unlink(file)

		print "Done. There are currently", goodFiles, "approved files in this repository"

def usage():
	print '\nThis module accepts the following options :\n'
	print '	-l : find the latest package list and use is as parameter for other operations'
	print '	-u : update the channel'
	print '	-c : cleanup channel directory\n'
	print '	--partial : valid with -u, will not warn about missing rpms'
	print '	--yes : valid with -c, will assume files can be deleted without confirmation\n'
	print 'Unless -l is used, must receive a packagelist filename argument.\n'
	print 'Examples :\n'
	print '	nrh_repository_update.py -uc --partial --yes redhat-linux-i386-7.3.20030626135559'
	print '	nrh_repository_update.py -luc --yes\n'
	print 'Notes :\n'
	print '	All processing is ran in the current directory.'
	print '	If \'-uc\' are used together, channel will first be updated, then cleaned up.\n'


# We want to do this in case someone calls this on its own
if __name__ == '__main__':

	if not len(sys.argv) > 1:
		usage()
		sys.exit(1)

	USE_LATEST_PKGLIST=0
	UPDATE_REP=0
	CLEAN_REP=0

	try:
		optlist,args=getopt.getopt(sys.argv[1:],"luc",['help','partial','yes'])
	except getopt.error:
		print "Argument error"
		sys.exit(1)
	except:
		print "Argument parsing error"

	for i in optlist:
		if i[0] == '-l':
			USE_LATEST_PKGLIST=1
		if i[0] == '-u':
			UPDATE_REP=1
		if i[0] == '-c':
			CLEAN_REP=1
		if i[0] == '--help':
			usage()
			sys.exit(1)

	if USE_LATEST_PKGLIST:
		locate_latest_pkglist().run(optlist,args)

	if UPDATE_REP:
		update_repository().run(optlist,args)

	if CLEAN_REP:
		clean_repository().run(optlist,args)
